package util

import (
	"io"
	"os"
	"path/filepath"
	"sort"
	"strings"
)

func CreateDir(directory string) error {
	// 检查目录是否存在
	if _, err := os.Stat(directory); os.IsNotExist(err) {
		// 如果目录不存在，创建父目录
		err := os.MkdirAll(directory, os.ModePerm)
		if err != nil {
			return err
		}
	}
	return nil
}

// CreateParentDir 存在不创建，不存在则创建父目录
func CreateParentDir(filePath string) error {
	if err := os.MkdirAll(filepath.Dir(filePath), os.ModePerm); err != nil {
		return err
	}
	return nil
}

// RemoveDir 递归删除目录及其下的所有文件和子目录
func RemoveDir(dirPath string) error {
	// 打开目录
	dir, err := os.Open(dirPath)
	if err != nil {
		return err
	}
	defer dir.Close()
	// 读取目录下的文件和子目录
	fileInfos, err := dir.Readdir(-1)
	if err != nil {
		return err
	}
	// 遍历文件和子目录
	for _, fileInfo := range fileInfos {
		path := filepath.Join(dirPath, fileInfo.Name())

		if fileInfo.IsDir() {
			// 如果是子目录，递归调用removeDir删除子目录及其下的文件和子目录
			if err = RemoveDir(path); err != nil {
				return err
			}
		} else {
			// 如果是文件，直接删除文件
			if err = os.Remove(path); err != nil {
				return err
			}
		}
	}
	// 删除目录本身
	if err = os.Remove(dirPath); err != nil {
		return err
	}
	return nil
}

func WriteFile(sourceContent string, targetFilePath string) error {
	if err := CreateFile(targetFilePath); err != nil {
		return err
	}
	if err := os.WriteFile(targetFilePath, []byte(sourceContent), 0644); err != nil {
		return err
	}
	return nil
}

// CreateFile 存在则覆盖，不存在则创建文件
func CreateFile(filePath string) error {
	if err := CreateParentDir(filePath); err != nil {
		return err
	}
	// 创建文件，如果文件已存在则覆盖
	file, err := os.Create(filePath)
	if err != nil {
		return err
	}
	defer func(file *os.File) {
		err := file.Close()
		if err != nil {

		}
	}(file)
	return nil
}
func ReadFile(filePath string) (string, error) {
	// 1 打开文件
	file, err := os.Open(filePath)
	if err != nil {
		return "", err
	}
	defer file.Close()

	// 2 读取文件内容
	content, err := io.ReadAll(file)
	if err != nil {
		return "", err
	}
	return string(content), err
}

func ListAbsolutePathWithSuffixAndSort(dir string, suffix string) ([]string, error) {
	var result []string
	if !strings.HasSuffix(dir, "/") {
		dir = dir + "/"
	}
	files, err := os.ReadDir(dir)
	if err != nil {
		return nil, err
	}
	for _, file := range files {
		if strings.HasSuffix(file.Name(), suffix) {
			result = append(result, dir+file.Name())
		}
	}
	// 根据文件名进行升序排序
	sort.Slice(result, func(i, j int) bool {
		return filepath.Base(result[i]) < filepath.Base(result[j])
	})
	return result, nil
}

func DeleteFile(path string) error {
	// 检查文件是否存在
	if _, err := os.Stat(path); err == nil {
		// 文件存在，执行删除操作
		err = os.Remove(path)
		if err != nil {
			return err
		}
	}
	return nil
}
func ListAbsolutePathAndSort(dir string) ([]string, error) {
	var result []string
	if !strings.HasSuffix(dir, "/") {
		dir = dir + "/"
	}
	files, err := os.ReadDir(dir)
	if err != nil {
		return result, err
	}
	for _, file := range files {
		result = append(result, dir+file.Name())
	}

	// 根据文件名进行升序排序
	sort.Slice(result, func(i, j int) bool {
		return filepath.Base(result[i]) < filepath.Base(result[j])
	})
	return result, nil
}
